/*
   Thanks to Original Author: ryand1011 (https://github.com/ryand1011)
   Edited by Ananords - https://github.com/ananords
   Edited by mattb_138 - https://www.instructables.com/member/mattb_138/

   See: https://github.com/miguelbalboa/rfid/tree/master/examples/rfid_write_personal_data

   Uses MIFARE RFID card using RFID-RC522 reader
   Uses MFRC522 - Library
   -----------------------------------------------------------------------------------------
               MFRC522      Arduino       Arduino   Arduino    Arduino          Arduino
               Reader/PCD   Uno/101       Mega      Nano v3    Leonardo/Micro   Pro Micro
   Signal      Pin          Pin           Pin       Pin        Pin              Pin
   -----------------------------------------------------------------------------------------
   RST/Reset   RST          9             5         D9         RESET/ICSP-5     RST
   SPI SS      SDA(SS)      10            53        D10        10               10
   SPI MOSI    MOSI         11 / ICSP-4   51        D11        ICSP-4           16
   SPI MISO    MISO         12 / ICSP-1   50        D12        ICSP-1           14
   SPI SCK     SCK          13 / ICSP-3   52        D13        ICSP-3           15



  PS: IF THE PLAYER FAILS TO START WHEN THE SERIAL MONITOR IS NOT OPEN, TRY TO COMMENT OUT Serial.begin(115200);
*/

#include <SPI.h>
#include <MFRC522.h>
#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

#define RST_PIN         9           // Configurable, see typical pin layout above
#define SS_PIN          10          // Configurable, see typical pin layout above

const int playPauseButton = 2;  // used to play/pause and long press to skip to next song
const int sideBButton = 3;      // used to change to Side B mode and long press to shuffle songs
const int sideBLED = 8;         // indicates if in Side B mode
const byte volumePot = A0;
int prevVolume; 

boolean sideB = false;
const int shortPressTime = 500; // define how long you want a short press to be (in this case less than 500 milliseconds)
const int longPressTime = 500;  // define how long you want a long press to be (in this case more than 500 milliseconds)

byte volumeLevel = 0; //variable for holding volume level

boolean isPlaying = false;
int lastStatePlayPause = LOW;  // the previous state from the input pin
int currentStatePlayPause;     // the current reading from the input pin
unsigned long pressedTimePlayPause  = 0;
unsigned long releasedTimePlayPause = 0;
bool isPressingPlayPause = false;
bool isLongDetectedPlayPause = false;

int lastStateSideB = LOW;  // the previous state from the input pin
int currentStateSideB;     // the current reading from the input pin
unsigned long pressedTimeSideB  = 0;
unsigned long releasedTimeSideB = 0;
bool isPressingSideB = false;
bool isLongDetectedSideB = false;

MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance

SoftwareSerial mySoftwareSerial(5, 6); // RX, TX

DFRobotDFPlayerMini myDFPlayer;
void printDetail(uint8_t type, int value);



//*****************************************************************************************//
void setup() {

  Serial.begin(115200);                                           // Initialize serial communications with the PC, COMMENT OUT IF IT FAILS TO PLAY WHEN DISCONNECTED FROM PC
  mySoftwareSerial.begin(9600);
  SPI.begin();                                                  // Init SPI bus
  mfrc522.PCD_Init();                                              // Init MFRC522 card



  pinMode(playPauseButton, INPUT_PULLUP);
  pinMode(sideBButton, INPUT_PULLUP);
  pinMode(sideBLED, OUTPUT);

  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));

  if (!myDFPlayer.begin(mySoftwareSerial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
  }
  Serial.println(F("DFPlayer Mini online. Place card on reader to play a specific song"));

  //myDFPlayer.volume(15);  //Set volume value. From 0 to 30 
  volumeLevel = map(analogRead(volumePot), 0, 1023, 0, 30);   //scale the pot value and volume level
  myDFPlayer.volume(volumeLevel);
  prevVolume = volumeLevel;

  //----Set different EQ----
  //  myDFPlayer.EQ(DFPLAYER_EQ_NORMAL);
  //  myDFPlayer.EQ(DFPLAYER_EQ_POP);
  //  myDFPlayer.EQ(DFPLAYER_EQ_ROCK);
  //  myDFPlayer.EQ(DFPLAYER_EQ_JAZZ);
  //  myDFPlayer.EQ(DFPLAYER_EQ_CLASSIC);
  myDFPlayer.EQ(DFPLAYER_EQ_BASS);

}



//*****************************************************************************************//
void loop() {


  volumeLevel = map(analogRead(volumePot), 0, 1023, 0, 30);   //scale the pot value and volume level


  if (prevVolume != volumeLevel){
  myDFPlayer.volume(volumeLevel);
  }
    prevVolume = volumeLevel;

  // read the state of the switch/button:
  currentStateSideB = digitalRead(sideBButton);

  if(lastStateSideB == HIGH && currentStateSideB == LOW) {        // button is pressed
    pressedTimeSideB = millis();
    isPressingSideB = true;
    isLongDetectedSideB = false;
  } else if(lastStateSideB == LOW && currentStateSideB == HIGH) { // button is released
    isPressingSideB = false;
    releasedTimeSideB = millis();

    long pressDurationSideB = releasedTimeSideB - pressedTimeSideB;

    if( pressDurationSideB < shortPressTime ) {                  // A short press will set the mode to read the cards (Side A or Side B) - LED will light up to indicate Side B
        sideB = !sideB;                                          // and the relevant Side will be displayed in the Serial
        digitalWrite(sideBLED, sideB);
      if (sideB == true) {
          Serial.println("Reading songs on Side B");
        } else if (sideB == false) {
          Serial.println("Reading songs on Side A");
        }
  }}

  if(isPressingSideB == true && isLongDetectedSideB == false) {
    long pressDurationSideB = millis() - pressedTimeSideB;

    if( pressDurationSideB > longPressTime ) {                  // A long press will shuffle all the songs on the SD card
      isLongDetectedSideB = true;
    myDFPlayer.randomAll();
    Serial.println("Shuffle Play");
    isPlaying = true;
    delay(1000);
    }
  }

  // save the the last state
  lastStateSideB = currentStateSideB;



  // read the state of the switch/button:
  currentStatePlayPause = digitalRead(playPauseButton);

  if(lastStatePlayPause == HIGH && currentStatePlayPause == LOW) {        // button is pressed
    pressedTimePlayPause = millis();
    isPressingPlayPause = true;
    isLongDetectedPlayPause = false;
  } else if(lastStatePlayPause == LOW && currentStatePlayPause == HIGH) { // button is released
    isPressingPlayPause = false;
    releasedTimePlayPause = millis();
  
    long pressDurationPlayPause = releasedTimePlayPause - pressedTimePlayPause;

    if( pressDurationPlayPause < shortPressTime ) {                      // A short press will pause the song that's currently playing or start playing where it left off
    if (isPlaying) {
      myDFPlayer.pause();
      isPlaying = false;
      Serial.println("Paused..");
    }
    else {
      isPlaying = true;
      myDFPlayer.start();
      Serial.println("Playing..");
    }
    delay(500);}}
  

  if(isPressingPlayPause == true && isLongDetectedPlayPause == false) {  // A long press will skip the current song and play the next song
    long pressDurationPlayPause = millis() - pressedTimePlayPause;

    if( pressDurationPlayPause > longPressTime ) {
      isLongDetectedPlayPause = true;
    myDFPlayer.next();
    Serial.println("Playing Next Song");
    isPlaying = true;
    delay(1000);
    }
  }

  // save the the last state
  lastStatePlayPause = currentStatePlayPause;


  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  MFRC522::MIFARE_Key key;
  for (byte i = 0; i < 6; i++) key.keyByte[i] = 0xFF;

  //some variables we need
  byte block;
  byte len;
  MFRC522::StatusCode status;


  //-------------------------------------------

  // Reset the loop if no new card present on the sensor/reader. This saves the entire process when idle.
  if ( mfrc522.PICC_IsNewCardPresent()) {



    // Select one of the cards
    if ( ! mfrc522.PICC_ReadCardSerial()) {
      return;
    }

    Serial.println(F("**Card Detected:**"));

    //-------------------------------------------

    mfrc522.PICC_DumpDetailsToSerial(&(mfrc522.uid)); //dump some details about the card

    //mfrc522.PICC_DumpToSerial(&(mfrc522.uid));      //uncomment this to see all blocks in hex

    //-------------------------------------------

    Serial.print(F("Number: "));


    //---------------------------------------- GET NUMBER AND PLAY THE SONG

    byte buffer2[18];
    block = 1;
    len = 18;


    status = mfrc522.PCD_Authenticate(MFRC522::PICC_CMD_MF_AUTH_KEY_A, 1, &key, &(mfrc522.uid)); //line 834
    if (status != MFRC522::STATUS_OK) {
      Serial.print(F("Authentication failed: "));
      Serial.println(mfrc522.GetStatusCodeName(status));
      return;
    }

    status = mfrc522.MIFARE_Read(block, buffer2, &len);
    if (status != MFRC522::STATUS_OK) {
      Serial.print(F("Reading failed: "));
      Serial.println(mfrc522.GetStatusCodeName(status));
      return;
    }


    //PRINT NUMBER
    String number = "";

    for (uint8_t i = 0; i < 16; i++)
    {
   number += (char)buffer2[i];

    }
    number.trim();
    Serial.print(number);
    

    //PLAY SONG

/*
 * Side B is achieved by adding or subtracting numbers to the card number that is read by the RC522.
 * In this case, my original album numbers were 89 - 99, and I had numbered folders on my SD Card
 * and placed the tracks in those folders.  The remaining songs were put in a folder and numbered 0001 - 0088,
 * and 0100.  
 * 
 * So I decided to use folder numbers 78 - 88 for my next lot of albums and 0101 and above for the 'single' tracks.
 * 
 * For the albums, the reader will read card 91 for example, then subtract 11 for a result of 80.  It will then 
 * play the album tracks in folder numbered 80 on the SD Card.
 * 
 * The singles work the same way.  If we scan card 49 for example, it will add 100 for a result of 149 and play 
 * the track 0149 in the main folder on the SD Card.
 * 
 * This all happens in the code below:
 */

if (sideB == true) {
    if ((number.toInt()>=89) && (number.toInt()<99)) {
    myDFPlayer.loopFolder(number.toInt()-11);            // change the 11 to suit the number of albums you've chosen to use
    } else {
     myDFPlayer.playLargeFolder(1,number.toInt()+100);  // change the 100 to suit the number of cards you're using.  I had 100 cards originally.  If you had 50, then you would only need to put in 50.  Just make sure the 
    isPlaying = true;}                                  // track numbers on your SD Card match up.  EG: If you have 50 cards, then the song on 'Side B' for the card number 12 will be 12 + 50 = 62 - file name starts 0062.
} else {
    if ((number.toInt()>=89) && (number.toInt()<99)) {  // Otherwise, if 'Side B' isn't selected, then just play normal numbers as coded on the cards.
    myDFPlayer.loopFolder(number.toInt());              // number album folders with the same number as the card associated to it and then loop the songs in that folder
    } else {
     myDFPlayer.playLargeFolder(1,number.toInt());      // number single songs with 4 digits and place in them in folder 01 on the SD Card, this is how the system identifies what track to play
    isPlaying = true;}
}


    //----------------------------------------

    Serial.println(F("\n**End Reading**\n"));

    delay(1000); //change value if you want to read cards faster

    mfrc522.PICC_HaltA();
    mfrc522.PCD_StopCrypto1();
  }
}
